"""
This module implements a portal-managed Audio class.  It is based on
Zope's built-in File object and the Audio product, so it's Portal-friendly.
"""

ADD_CONTENT_PERMISSION = 'Add portal content'


__doc__ = """CMF Audio product module."""
__version__ = '$Revision: 0.6 $'[11:-2]  


from Globals import HTMLFile, HTML
from Products.CMFCore.PortalContent import PortalContent
#from Discussions import Discussable
import Globals
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.WorkflowCore import WorkflowAction

from Products.CMFDefault.File import File
from Products.Audio.Audio import Audio


from OFS.PropertyManager import PropertyManager

import OFS.Image
import Products.Audio.mp3info
import string

# Factory type information -- makes CMFAudio objects play nicely
# with the Types Tool (portal_types)
factory_type_information =  ({'id': 'Audio',
     'meta_type': 'Portal Audio',
     'description': """Audio file, preferably MP3""",
     'product': 'CMFAudio',
     'icon': 'mp3.gif',
     'content_icon': 'mp3.gif',
     'factory': 'addFile',
     'immediate_view': 'file_edit_form',
     'actions': 
            ({'name': 'View',
              'action': 'audioView',
              'permissions': ('View',)},
            {'name': 'Download',
             'action': '',
             'permissions': ('View',)},
            {'name': 'Upload',
             'action': 'file_edit_form',
             'permissions': ('Modify portal content',)},
            {'name': 'Song Data',
             'action': 'audioEdit',
             'permissions': ('Modify portal content',)},
            {'name': 'Metadata',
             'action': 'metadata_edit_form',
             'permissions': ('Modify portal content',)},
        
            ),
       },)

#addFile modified from CMFDefault.File

def addFile( self
           , id
           , title=''
           , file=''
           , content_type='audio/unknown'
           , precondition=''
           , subject=()
           , description=''
           , contributors=()
           , effective_date=None
           , expiration_date=None
           , format='audio/unknown'
           , language='en-US'
           , rights=''
           ):
    """
    Add an Audio File
    """

    # cookId sets the id and title if they are not explicity specified
    id, title = OFS.Image.cookId(id, title, file)

    self=self.this()

    # Instantiate the object and set its description.
    fobj = CMFAudio( id, title, file, content_type, precondition, subject
               , description, contributors, effective_date, expiration_date
               , format, language, rights
               )
    
    # Add the File instance to self
    self._setObject(id, fobj)

    # 'Upload' the file.  This is done now rather than in the
    # constructor because the object is now in the ZODB and
    # can span ZODB objects.
    self._getOb(id).manage_upload(file)
    



class CMFAudio(Audio, File):
    """ CMFAudio: A Filetype with stuff to extract audio metadata, CMF friendly """
    
    meta_type = 'Portal Audio'

    __ac_permissions__ = File.__ac_permissions__

    def __init__(self
                , id
                , title=''
                , file=''
                , content_type='audio/unknown'
                , precondition=''
                , subject=()
                , description=''
                , contributors=()
                , effective_date=None
                , expiration_date=None
                , format='audio/unknown'
                , language='en-US'
                , rights=''
                ):
        """ Initialize a CMFAudio """
        #order is important here!    
        Audio.__init__(self, id, title, file
                               , content_type, precondition)
                               
        File.__init__( self, id, title, file
            , content_type, precondition, subject, description, contributors
            , effective_date, expiration_date, format, language, rights)


#replace searchable text method

    
    def SearchableText(self):
        """
        SeachableText is used for full text seraches of a portal.  It
        should return a concatanation of all useful text.
        """
        return string.join((self.title, self.description, self.songname
            , self.artist, self.album, self.genre, self.filetype))
            
#replace File.download method, as it forces inheritance from OFS.File

    def download(self, REQUEST, RESPONSE):
        """Download this item.
        
        Calls index_html to perform the actual transfer after
        first setting Content-Disposition to suggest a filename.
        
        This method is deprecated, use the URL of this object itself. Because
        the default view of a File object is to download, rather than view,
        this method is obsolete. Also note that certain browsers do not deal
        well with a Content-Disposition header.

        """

        RESPONSE.setHeader('Content-Disposition',
                           'attachment; filename=%s' % self.getId())
        return self.index_html(REQUEST, RESPONSE)

#need to put these in, to override inheritance order on the two methods    
    def manage_afterAdd(self, item, container):
        """Both of my parents have an afterAdd method"""
        File.manage_afterAdd(self, item, container)

    def manage_beforeDelete(self, item, container):
        """Both of my parents have a beforeDelete method"""
        File.manage_beforeDelete(self, item, container)
        
  
#override Audio's update_data in order to push song data into DublinCore
    def update_data(self, data, content_type=None, size=None):
        Audio.update_data(self, data, content_type, size)
        # push song data into DublinCore for convenience
        self._editMetadata( title = self.songname
                         , description =  self.artist + '\n' \
             + self.album + '\n' + (self.year and ('Copyright (c)(P)' + self.year))
	                     , contributors = self.artist
						 , format = self.content_type
                         , rights = self.copyright
						 )

    
Globals.default__class_init__(CMFAudio)

# PortalContent classes need to be registered.  This installs the
# class's factory method and icon.
##from Products.CMFCore.register import registerPortalContent
##registerPortalContent(CMFAudio,                    # The class to be registered
##                      constructors=addFile, # It's factory (constructor)
##                      action='Wizards/File',   # The URL of it's add interface
##                      icon="mp3.gif",         # Filename to take icon from
##                      productGlobals=globals())
